###########################################################################
#
# ExcelPlugin.pm -- plugin for importing Microsoft Excel files.
#  (basic version supports versions 95 and 97)
#  (through OpenOffice extension, supports all contempoary formats)
#
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 2002 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

package ExcelPlugin;

use strict;
no strict 'refs'; # allow filehandles to be variables and viceversa
no strict 'subs';
use gsprintf 'gsprintf';

use AutoloadConverterScripting;

@ExcelPlugin::ISA = ('AutoloadConverterScripting');


my $arguments = 
    [ { 'name' => "process_exp",
	'desc' => "{BasePlugin.process_exp}",
	'type' => "regexp",
	'reqd' => "no",
	'deft' => "&get_default_process_exp(\$self)"  # delayed (see below)
	}
      ];

my $options = { 'name'     => "ExcelPlugin",
		'desc'     => "{ExcelPlugin.desc}",
		'abstract' => "no",
		'inherits' => "yes",
		'srcreplaceable' => "yes", # Source docs in Excel format can be replaced with GS-generated html
		'args'     => $arguments };

sub new {
    my ($class) = shift (@_);
    my ($pluginlist,$inputargs,$hashArgOptLists) = @_;
    push(@$pluginlist, $class);
    
    push(@{$hashArgOptLists->{"ArgList"}},@{$arguments});
    push(@{$hashArgOptLists->{"OptList"}},$options);

    my $self 
	= new AutoloadConverterScripting("OpenOfficeConverter",$pluginlist, 
					 $inputargs, $hashArgOptLists);

    # plugin's process_exp can only be correctly determined once autoloading
    # has taken place
    my $plug_options = $self->{'option_list'}->[0];
    my $plug_args = $plug_options->{'args'};
    
    foreach my $a (@$plug_args) {
	# consider changing this to search for all values that are
	# tagged as 'deft-delayed' = 1 ?!?

	if ($a->{'name'} eq "process_exp") {
	    my $eval_expr = $a->{'deft'};
	    $a->{'deft'} = eval "$eval_expr";

	    # Now see if process_exp needs updating
	    my $process_exp = $self->{'process_exp'};
	    if (!$self->{'info_only'} && ($process_exp eq $eval_expr)) {
		# process_exp is only defined if not 'info_only'
		#
		# if it does exist and it equals the unevaluated $eval_expr
		# then it was set to the default (rather than overriden by
		# the collect.cfg file)

		$self->{'process_exp'} = $a->{'deft'};
	    }
	}
    }
    
    if ($self->{'info_only'}) {
	# don't worry about any options etc
	return bless $self, $class;
    }

    $self->{'filename_extension'} = "xls";
    $self->{'file_type'} = "Excel";

    my $outhandle = $self->{'outhandle'};

    # check convert_to
    if ($self->{'convert_to'} eq "auto") {
	$self->{'convert_to'} = "html";
    }

    $self = bless $self, $class;
    # set convert_to_plugin and convert_to_ext
    $self->set_standard_convert_settings();

    my $secondary_plugin_name = $self->{'convert_to_plugin'};
    my $secondary_plugin_options = $self->{'secondary_plugin_options'};

    if (!defined $secondary_plugin_options->{$secondary_plugin_name}) {
	$secondary_plugin_options->{$secondary_plugin_name} = [];
    }
    my $specific_options = $secondary_plugin_options->{$secondary_plugin_name};

    push(@$specific_options,"-extract_language") if $self->{'extract_language'};
    push(@$specific_options, "-file_rename_method", "none");

    if ($secondary_plugin_name eq "HTMLPlugin") {
	push(@$specific_options, "-processing_tmp_files");
    }
    
    $self->load_secondary_plugins($class,$secondary_plugin_options,$hashArgOptLists);
    return $self;    
}


sub convert_post_process_old
{
    my $self = shift (@_);
    my ($conv_filename) = @_;
    
    my $outhandle=$self->{'outhandle'};
    
    my ($language, $encoding) = $self->textcat_get_language_encoding ($conv_filename);
    
    # read in file ($text will be in utf8)
    my $text = "";
    $self->read_file ($conv_filename, $encoding, $language, \$text);
    
    # turn any high bytes that aren't valid utf-8 into utf-8.
    #unicode::ensure_utf8(\$text);
    
    # Write it out again!
    #$self->utf8_write_file (\$text, $conv_filename);
}

sub get_default_process_exp {
    my $self = shift (@_);

    if ($self->{'scripting_ext_working'}) {
	return q^(?i)\.(xls|xlsx|ods)$^;
    }

    return q^(?i)\.xls$^;
}
    

1;
